/*	
 *	Pollin Electronic GmbH
 *	Max-Pollin-Strae 1			
 *	85104 Pfrring 
 *	http://www.pollin.de
 *
 *	Project:	Synthesizer ver.1.0
 *  Version:	ver.1.0
 *  Filename:	main.c 
 *
 *  Copyright (c) 2017 Pollin Electronic GmbH. All rights reserved.
 *
 *  Bitte beachten Sie, dass dieses Programm nur uneingeschrnkt fr nicht kommerzielle Zwecke 
 *  genutzt werden darf. Ich bernehme keine Garantie fr die einwandfreie Funktion dieses Pro- 
 *  gramms und bernehme dadurch auch keine Haftung fr Folgeschden die Aufgrund der Benutzung
 *  dieser Daten entstehen. Dieses gilt auch dann, wenn ich ber einen solchen Schaden unter- 
 *	richtet worden bin. 
 *
 *  Mikrocontroller:			ATtiny 2313
 *						   ----------------------
 *				   		  |                      |
 *		  RESET	<---PA2---|1(RESET)		   VCC 20|
 *			C1	<---PD0---|2(RXD)		  (SCK)19|---PB7-->
 *			D1	<---PD1---|3(TXD)        (MISO)18|---PB6-->	LED Grn / Sgezahn
 *	   		E1	<---PA1---|4(XTAL2)      (MOSI)17|---PB5--> LED Rot / Envelope
 * 			F1	<---PA2---|5(XTAL1)      (OC1B)16|---PB4--> Envelopetaster
 * 	   		G1	<---PD2---|6(INT0)       (OC1A)15|---PB3--> Lautsprecher
 *			A1	<---PD3---|7(INT1)       (OC0A)14|---PB2--> Envelopausgang
 *			H1	<---PD4---|8(T0)         (AIN1)13|---PB1--> Sgezahntaster
 *			C2	<---PD5---|9(OC0B)       (AIN0)12|---PB0--> #/B
 *						  |10 GND	      (ICP)11|---PD6--> Octave
 *						   ----------------------
 *
 *  Fuse-Bits:  --- fr internen 8 MHz Takt mit Vorteiler 8 ----
 *
 *             --- PonyProg-Setting ---
 *  CKDIV8/CKOUT/CKSEL3..1   [1][0] [1011]
 *                                             
 *             SUT1..0     [0][1]               
 *
 *  Frequenz:   1.000.000 Hz
 *
 *  Mit dem 16Bit Timers des ATtiny werden die Grundtne erzeugt. Die Sgezahnfunkion welche mit
 *  dem Taster ein und ausgeschaltet werden kann basiert auf den berechneten Frequenzern der 
 *  Grundtne (siehe main.h). Die Sgezahnausgabe erfolgt mit dem 8bit Timer welcher den OCR1A 
 *  Ausgang (PB3) des 16Bit Timers manuell toggelt. 
 *  Die Envelopefunktion wird mit dem Watchdog der als Timer initialisiert ist erzeugt. 
 *  In der Mainschleife werden die Tasterbettigungen gezhlt und im Watchdoghandler ver-
 *  arbeitet. Nach 2-4-6-8 oder 10maligen Aufrufen des Handlers werden die LED und der Ausgangs-
 *  port getoggelt. 
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include "main.h"

volatile int8_t tastenaktuell = 0x00;	//c'' h' a' g' f' e' d' c'
volatile int8_t tastensonder = 0x00;	//Envelope, Oktave, #/B, Sgezahn

volatile char envelope = 0;				//Envelopezhler
volatile char zaehlere = 0;				//Zhler fr Blinkfrequenz Envelope
volatile char sagezahn = 0;				//Zhler fr Sgezahnerzeugung

void init()
{
	/*16 Bit Timer*/
	TCCR1A = 0b01000000;	//Toggle OC1A 
	TCCR1B = 0b00001001;	//CTC, Kein Vorteiler 
	OCR1A = 0;				
	
	/* 8 Bit Timer */	
//	TCCR0A = (1<<WGM01);	//CTC
//	TCCR0B = (1<<CS01);		//Vorteiler 8
//	TIMSK |= (1<<OCIE0A);	//Interrupt aktivieren
	
	/*Taster Eingang & Pullup*/
	DDRC11 &= ~(1<<BITC11);	
	DDRD11 &= ~(1<<BITD11);
	DDRE11 &= ~(1<<BITE11);
	DDRF11 &= ~(1<<BITF11);
	DDRG11 &= ~(1<<BITG11);
	DDRA11 &= ~(1<<BITA11);
	DDRH11 &= ~(1<<BITH11);
	DDRC22 &= ~(1<<BITC22);
	DDRENVELOPE &= ~(1<<BITENVELOPE);
	DDROKTAVE &= ~(1<<BITOKTAVE);
	DDRKREUZB &= ~(1<<BITKREUZB);
	DDRSAGEZAHN &= ~(1<<BITSAGEZAHN);

	PORTENVELOPE |= (1<<BITENVELOPE);
	PORTSAGEZAHN |= (1<<BITSAGEZAHN);

	/*Lautsprecherausgang*/
	DDRLAUT |= (1<<BITLAUT);		//OCR1A
	DDRENVAUS |= (1<<BITENVAUS);	//Envelopeausgang

	/* LED */
	DDRLEDGR |= (1<<BITLEDGR);
	DDRLEDRO |=	(1<<BITLEDRO);
	
	/* Watchdog als Timer */
	WDTCSR = 0b01000000;
	
	/* Globale Interrupts */
	sei();
}

void readtasten()
{
	tastensonder = tastensonder<<4;		//alten Tastenzustand speichern
	
	if(!(PINC11 & (1<<BITC11)))
		tastenaktuell |= 0x1;
	else
		tastenaktuell &= ~0x1;
		
	if(!(PIND11 & (1<<BITD11)))
		tastenaktuell |= 0x2;
	else
		tastenaktuell &= ~0x2;
	
	if(!(PINE11 & (1<<BITE11)))
		tastenaktuell |= 0x4;
	else
		tastenaktuell &= ~0x4;
		
	if(!(PINF11 & (1<<BITF11)))
		tastenaktuell |= 0x8;
	else
		tastenaktuell &= ~0x8;
		
	if(!(PING11 & (1<<BITG11)))
		tastenaktuell |= 0x10;
	else
		tastenaktuell &= ~0x10;
		
	if(!(PINA11 & (1<<BITA11)))
		tastenaktuell |= 0x20;
	else
		tastenaktuell &= ~0x20;
		
	if(!(PINH11 & (1<<BITH11)))
		tastenaktuell |= 0x40;
	else
		tastenaktuell &= ~0x40;
		
	if(!(PINC22 & (1<<BITC22)))
		tastenaktuell |= 0x80;
	else 
		tastenaktuell &= ~0x80;
		
	if(!(PINENVELOPE & (1<<BITENVELOPE)))
		tastensonder |= 0x1;
	else
		tastensonder &= ~0x1;
			
	if(!(PINOKTAVE & (1<<BITOKTAVE)))
		tastensonder |= 0x2;
	else
		tastensonder &= ~0x2;
	
	if(!(PINKREUZB & (1<<BITKREUZB)))
		tastensonder |= 0x4;
	else
		tastensonder &= ~0x4;
	
	if(!(PINSAGEZAHN & (1<<BITSAGEZAHN)))
		tastensonder |= 0x8;
	else
		tastensonder &= ~0x8;	
}

int main(void)
{
	char faktor = 1;		//zur Berechnung der Oktave
	char saegezahn = 0;		//Sgezahn an/aus
	
	init();
	
    while (1) 
    {
		readtasten();		//Tastenzustand einlesen
		
		/* Sgezahn aktivieren */	
		if(OCR0A != 0)
			TIMSK |= (1<<OCIE0A);
		else
			TIMSK &= ~(1<<OCIE0A);
		
		/* Abfrage der Sgezahntaste */		
		if ((tastensonder & 0x8) && (!(tastensonder & 0x80)) && (saegezahn == 0))
		{
			saegezahn = 1;
			PORTLEDGR |= (1<<BITLEDGR);	//LED an
			
			TCCR1A = 0;					//16 Bit Timer aus
			TCCR1B = 0;
			
			TCCR0A = (1<<WGM01);		//8 Bit Timer ein
			TCCR0B = (1<<CS01);
			TIMSK &= ~(1<<OCIE0A);
		}
		else if((tastensonder & 0x8) && (!(tastensonder & 0x80)) && (saegezahn == 1))
		{
			saegezahn = 0;
			PORTLEDGR &= ~(1<<BITLEDGR);//LED aus
			
			TCCR0A = 0;					//8 Bit Timer aus 
			TCCR0B = 0;
			TIMSK = 0;
			
			TCCR1A = 0b01000000;		//16 Bit Timer ein
			TCCR1B = 0b00001001;	
			OCR1A = 0;
		}

		/* Abfrage der Oktave */
		if(tastensonder & 0x2)
			faktor = 2;
		else
			faktor = 1;
		
		/* Wenn Kreuz/B nicht gedrckt normaler Tonleiter */
		if(!(tastensonder & 0x4))
		{			
			if(tastenaktuell & 0x1)
			{
				OCR1A = 1974/faktor;	//C
				OCR0A = 49/faktor;
			}
			else if (tastenaktuell & 0x2)
			{
				OCR1A = 1759/faktor;	//D
				OCR0A = 44/faktor;
			}
			else if (tastenaktuell & 0x4)
			{
				OCR1A = 1567/faktor;	//E
				OCR0A = 39/faktor;
			}
			else if (tastenaktuell & 0x8)
			{
				OCR1A = 1479/faktor;	//F
				OCR0A = 37/faktor;
			}
			else if (tastenaktuell & 0x10)
			{
				OCR1A = 1318/faktor;	//G
				OCR0A = 33/faktor;
			}
			else if (tastenaktuell & 0x20)
			{
				OCR1A = 1174/faktor;	//A
				OCR0A = 29/faktor;
			}
			else if (tastenaktuell & 0x40)
			{
				OCR1A = 1046/faktor;	//H
				OCR0A = 26/faktor;
			}
			else if (tastenaktuell & 0x80)
			{
				OCR1A = 987/faktor;		//C
				OCR0A = 24/faktor;
			}
			else
			{
				OCR1A = 0;
				OCR0A = 0;
			}
		}

		/* Kreuz/B gedrckt */
		if(tastensonder & 0x4)
		{
			if(tastenaktuell & 0x1)
			{
				OCR1A = 1863 / faktor;	//CIS
				OCR0A = 47/faktor;
			}
			else if ((tastenaktuell & 0x4) || (tastenaktuell & 0x2))
			{
				OCR1A = 1660 / faktor;	//DIS
				OCR0A = 42/faktor;
			}
			else if (tastenaktuell & 0x8)
			{
				OCR1A = 1396 / faktor;	//FIS
				OCR0A = 35/faktor;
			}
			else if (tastenaktuell & 0x10)
			{
				OCR1A = 1242 / faktor;	//GIS
				OCR0A = 31/faktor;
			}
			else if ((tastenaktuell & 0x20) || (tastenaktuell & 0x40))
			{
				OCR1A = 1109 / faktor;	//AIS
				OCR0A = 28/faktor;
			}
			else if (tastenaktuell & 0x80)
			{
				OCR1A = 932 / faktor;	//CIS
				OCR0A = 23/faktor;
			}
			else
			{
				OCR1A = 0;
				OCR0A = 0;
			}
		}  

		/* Envelop */
		if ((tastensonder & 0x1) && (!(tastensonder & 0x10)))
		{
			envelope++;
			
			if(envelope == 6)
				envelope = 0;
		}	
	}
}

ISR (WDT_OVERFLOW_vect)
{
	zaehlere++;
			
	/* Envelope Blinken & Frequenz */
	if(envelope != 0)
	{
		if(envelope == 1 && zaehlere == 2)
		{
			PORTLEDRO ^= (1<<BITLEDRO);		//LED Blinken
			zaehlere = 0;
			PORTENVAUS ^= (1<<BITENVAUS);	//Lautstrke variieren
		}
		if(envelope == 2 && zaehlere == 4)
		{
			PORTLEDRO ^= (1<<BITLEDRO);
			zaehlere = 0;
			PORTENVAUS ^= (1<<BITENVAUS);
		}
		if(envelope == 3 && zaehlere == 6)
		{
			PORTLEDRO ^= (1<<BITLEDRO);
			zaehlere = 0;
			PORTENVAUS ^= (1<<BITENVAUS);
		}
		if(envelope == 4 && zaehlere == 8)
		{
			PORTLEDRO ^= (1<<BITLEDRO);
			zaehlere = 0;
			PORTENVAUS ^= (1<<BITENVAUS);
		}
		if(envelope == 5 && zaehlere == 10)
		{
			PORTLEDRO ^= (1<<BITLEDRO);
			zaehlere = 0;
			PORTENVAUS ^= (1<<BITENVAUS);
		}
	}
	else
	{
		zaehlere = 0;
		PORTLEDRO |= (1<<BITLEDRO);		//LED ein
		PORTENVAUS |= (1<<BITENVAUS);	//Ausgang High
	}
}


ISR (TIMER0_COMPA_vect)
{
	sagezahn++;
	
	switch (sagezahn)
	{
		case 1:
			PORTB |= (1<<PB3);	//Lautsprecherausgang
			break;
		case 2: 
			PORTB &= ~(1<<PB3);
			break;
		case 11: 
			PORTB |= (1<<PB3);
			break;
		case 13:
			PORTB &= ~(1<<PB3);
			break;
		case 21:
			PORTB |= (1<<PB3);
			break;
		case 23:
			PORTB &= ~(1<<PB3);
			break;
		case 31:
			PORTB |= (1<<PB3);
			break;
		case 34:
			PORTB &= ~(1<<PB3);
			break;
		case 41:
			PORTB |= (1<<PB3);
			break;
		case 45:
			PORTB &= ~(1<<PB3);
			break;
		case 51:
			PORTB |= (1<<PB3);
			break;
		case 56:
			PORTB &= ~(1<<PB3);
			break;
		case 61:
			PORTB |= (1<<PB3);
			break;
		case 67:
			PORTB &= ~(1<<PB3);
			break;
		case 71:
			PORTB |= (1<<PB3);
			break;
		case 78:
			PORTB &= ~(1<<PB3);
			break;
		case 81:
			PORTB |= (1<<PB3);
			break;
		case 89:
			PORTB &= ~(1<<PB3);
			break;
		case 91:
			PORTB |= (1<<PB3);
			break;
		case 100:
		{
			PORTB &= ~(1<<PB3);
			sagezahn = 0;
			break; 
		}
		default:
		break;
	}
	
}